"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const config_1 = require("../config");
const cursor_1 = require("./strategies/cursor");
const offset_1 = require("./strategies/offset");
const page_1 = require("./strategies/page");
class Paginator {
    init() {
        this.config = config_1.config.get("pagination");
    }
    getRouteOptions(request) {
        return request.route.settings.plugins.pagination || {};
    }
    containsPath(path, array) {
        return lodash_1.some(array, item => (item instanceof RegExp ? item.test(path) : item === path));
    }
    isValidRoute(request) {
        const options = this.getRouteOptions(request);
        const { include, exclude } = this.config.routes;
        const { method, path } = request.route;
        if (options.enabled) {
            return options.enabled;
        }
        if (!["get", "post"].includes(method)) {
            return false;
        }
        if (exclude && this.containsPath(path, exclude)) {
            return false;
        }
        if (include[0] === "*") {
            return true;
        }
        if (!this.containsPath(path, include)) {
            return false;
        }
        return true;
    }
    getPagination(request) {
        const routeDefaults = this.getRouteOptions(request).defaults || {};
        return routeDefaults.pagination ? routeDefaults.pagination : true;
    }
    onPreHandler(request, h) {
        if (this.isValidRoute(request)) {
            const routeDefaults = this.getRouteOptions(request).defaults || {};
            if (!this.getPagination(request)) {
                return h.continue;
            }
            if (this.config.strategy === "cursor") {
                return cursor_1.cursorStrategy.onPreHandler(request, h, routeDefaults);
            }
            if (this.config.strategy === "offset") {
                return offset_1.offsetStrategy.onPreHandler(request, h, routeDefaults);
            }
            if (this.config.strategy === "page") {
                return page_1.pageStrategy.onPreHandler(request, h, routeDefaults);
            }
        }
        return h.continue;
    }
    onPostHandler(request, h) {
        const statusCode = request.response.statusCode;
        const processResponse = this.isValidRoute(request) && statusCode >= 200 && statusCode <= 299 && this.getPagination(request);
        if (!processResponse) {
            return h.continue;
        }
        if (this.config.strategy === "cursor") {
            return cursor_1.cursorStrategy.onPostHandler(request, h);
        }
        if (this.config.strategy === "offset") {
            return offset_1.offsetStrategy.onPostHandler(request, h);
        }
        if (this.config.strategy === "page") {
            return page_1.pageStrategy.onPostHandler(request, h);
        }
        return h.continue;
    }
}
exports.paginator = new Paginator();
//# sourceMappingURL=paginator.js.map